@import 'frontend-toolkit';

.form-date {
  // Django split date-time widget
  .form-group-date {
    width: 120px;
  }
  .form-group-time {
    width: 100px;
  }
  @include media(mobile) {
    .form-group-date, .form-group-time {
      width: 40%;
    }
  }

  // Django select date widget
  .form-group-year-select {
    width: 100px;
  }
  .form-group-month-select {
    width: 150px;
  }
  .form-group-day-select {
    width: 100px;
  }
  .form-group-year-select, .form-group-month-select, .form-group-day-select {
    .form-control {
        width: 100%;
    }

    @include media(mobile) {
      width: 50%;
      margin-bottom: $gutter-half;
      clear: left;
    }
  }
}
